SystemOrganization addCategory: #'Tests-System-Settings'!

TestCase subclass: #PragmaCollectorTest
	instanceVariableNames: 'lastReceivedAnnounce clientClass settingsProviderClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tests-System-Settings'!

!PragmaCollectorTest methodsFor: 'running' stamp: 'alain.plantec 3/30/2009 11:33'!
newCollectorForKeyword: aSymbol
	clientClass compile: 'handlerFromPragma: prg ^ (prg keyword = #', aSymbol, ') ifTrue: [PragmaHandler pragma: prg]'.
	^ (PragmaCollector new client: clientClass new) reset.
! !

!PragmaCollectorTest methodsFor: 'test - pragma announcement' stamp: 'alain.plantec 3/22/2009 11:46'!
receiveAnnounce: anAnnounce
	lastReceivedAnnounce := anAnnounce

! !

!PragmaCollectorTest methodsFor: 'running' stamp: 'alain.plantec 3/30/2009 13:15'!
setUp
	clientClass := Object subclass: #CLIENTFORSETTINGSTESTS
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'SETTINGSTESTS'.
	settingsProviderClass := Object subclass: #SETTINGSPROVIDERFORSETTINGSTESTS
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'SETTINGSTESTS'.
! !

!PragmaCollectorTest methodsFor: 'running' stamp: 'alain.plantec 3/30/2009 13:11'!
tearDown
	super tearDown.
	Smalltalk removeClassNamed: clientClass name.
	Smalltalk removeClassNamed: settingsProviderClass name.
	(Smalltalk at: #CLIENTFORSETTINGSTESTS ifAbsent: []) ifNotNil: [:cls | cls removeFromSystem].
	clientClass := nil.
	SystemOrganization removeSystemCategory: 'SETTINGSTESTS'.
	lastReceivedAnnounce := nil
! !

!PragmaCollectorTest methodsFor: 'test - system notifications' stamp: 'alain.plantec 3/30/2009 13:13'!
testAddPragma
	| collector |
	collector := self newCollectorForKeyword: #gloup6.
	self assert: collector isEmpty.
	settingsProviderClass compile: 'gloupIt <gloup6>'.
	self assert: collector collected size = 1.
	collector do: [:handler | self assert: (handler keyword = #gloup6)].
	settingsProviderClass removeSelector: #gloupIt.
	self assert: collector isEmpty.
	collector release

! !

!PragmaCollectorTest methodsFor: 'test - system notifications' stamp: 'alain.plantec 3/30/2009 13:06'!
testAddRemoveClassWithPragma
	| collector |
	collector := self newCollectorForKeyword: #gloup7.
	self assert: collector isEmpty.
	('
Object subclass: #GLOUPSETTINGSFORTEST
	instanceVariableNames: ''''
	classVariableNames: ''Gloup''
	poolDictionaries: ''''
	category: ''SETTINGSTESTS''!!
!!GLOUPSETTINGSFORTEST class methodsFor: ''settings'' stamp: ''alain.plantec 3/27/2009 18:35''!!
gloup
	<gloup7>
	^ Gloup 
		ifNil: [Gloup := (SettingManager newSetting: ''Gloup2'')
						default: true] !! !!
') readStream fileIn.
	self assert: collector collected size = 1.
	collector do: [:handler | self assert: (handler keyword = #gloup7).
		self assert: (handler methodClass theNonMetaClass name = #GLOUPSETTINGSFORTEST).
		self assert: (handler selector = #gloup)].
	(Smalltalk at: #GLOUPSETTINGSFORTEST) removeFromSystem.
	self assert: collector isEmpty.
	collector release

! !

!PragmaCollectorTest methodsFor: 'test - collecting basic' stamp: 'alain.plantec 4/7/2009 23:34'!
testCollectAllPragmas
	| all collector |
	"accept all pragmas"
	clientClass compile: 'handlerFromPragma: prg ^ PragmaHandler pragma: prg'.
	collector := PragmaCollector new client: clientClass new.
	collector reset.
	all := PragmaCollector allSystemPragmas asSet.
	self assert: (collector collected collect:[:c | c pragma]) asSet = all.
	clientClass compile: 'handlerFromPragma: prg ^ nil'.
	collector reset.
	self assert: (collector isEmpty).
	collector release.
	"No client -> takes all pragmas"
	collector := PragmaCollector new.
	collector reset.
	self assert: (collector collected collect:[:c | c pragma]) asSet = all.
	collector release.



	
	

! !

!PragmaCollectorTest methodsFor: 'test - collecting basic' stamp: 'alain.plantec 3/30/2009 11:39'!
testCollectNoPragmas
	| collector |
	"the requestor pragma holder"
	clientClass compile: 'handlerFromPragma: prg ^ nil'.
	collector := PragmaCollector new client: clientClass new.
	collector reset.
	self assert: (collector isEmpty).
	collector release

! !

!PragmaCollectorTest methodsFor: 'test - collecting basic' stamp: 'alain.plantec 3/30/2009 13:15'!
testCollectSpecificPragmas
	"the requestor pragma holder"
	| collector |
	collector := self newCollectorForKeyword: #gloup1.
	settingsProviderClass compile: 'gloupIt <gloup1>'.
	collector reset.
	self assert: (collector collected size = 1).
	collector do: [:handler | self assert: (handler keyword = #gloup1)].
	settingsProviderClass removeSelector: #gloupIt.
	self assert: (collector isEmpty).
	collector release



! !

!PragmaCollectorTest methodsFor: 'test - pragma announcement' stamp: 'alain.plantec 4/13/2009 07:25'!
testPragmaAdded
	| collector |
	collector := self newCollectorForKeyword: #gloup2.
	collector whenChangedSend: #receiveAnnounce: to: self.
	lastReceivedAnnounce := nil.
	settingsProviderClass compile: 'gloupIt <gloup2>'.
	self
		assert: ((lastReceivedAnnounce isKindOf: PragmaAnnouncement)
				and: [lastReceivedAnnounce isHandlerAdded]).
	self assert: collector collected size = 1. 
	self assert: collector collected first == lastReceivedAnnounce handler.
	collector unsubscribe: self.
	settingsProviderClass removeSelector: #gloupIt.
	self assert: collector isEmpty.
	collector release! !

!PragmaCollectorTest methodsFor: 'test - pragma announcement' stamp: 'alain.plantec 4/13/2009 07:26'!
testPragmaRemoved
	| oldHandler collector | 
	collector := self newCollectorForKeyword: #gloup4.
	settingsProviderClass compile: 'gloupIt <gloup4>'.
	self assert: collector collected size = 1.
	oldHandler := collector collected first.
	collector whenChangedSend: #receiveAnnounce: to: self.
	lastReceivedAnnounce := nil.
	settingsProviderClass removeSelector: #gloupIt.
	self
		assert: ((lastReceivedAnnounce isKindOf: PragmaAnnouncement)
				and: [lastReceivedAnnounce isHandlerRemoved]).
	self assert: (collector isEmpty).
	self assert: (oldHandler == lastReceivedAnnounce handler).
	collector unsubscribe: self.
	collector release




! !

!PragmaCollectorTest methodsFor: 'test - pragma announcement' stamp: 'alain.plantec 4/13/2009 07:53'!
testPragmaRemoved2
	| oldHandler collector | 
	collector := self newCollectorForKeyword: #gloup4.
	settingsProviderClass compile: 'gloupIt <gloup4>'.
	self assert: collector collected size = 1.
	oldHandler := collector collected first.
	collector whenChangedSend: #receiveAnnounce: to: self.
	lastReceivedAnnounce := nil.
	settingsProviderClass compile: 'gloupIt "no more pragma here"'.
	self
		assert: ((lastReceivedAnnounce isKindOf: PragmaAnnouncement)
				and: [lastReceivedAnnounce isHandlerRemoved]).
	self assert: (collector isEmpty).
	self assert: (oldHandler == lastReceivedAnnounce handler).
	collector unsubscribe: self.
	collector release




! !

!PragmaCollectorTest methodsFor: 'test - pragma announcement' stamp: 'alain.plantec 3/30/2009 13:12'!
testPragmaSubscription
	| oldLastReceivedAnnounce collector |
	collector := self newCollectorForKeyword: #gloup3.
	collector whenChangedSend: #receiveAnnounce: to: self.
	lastReceivedAnnounce := nil.
	settingsProviderClass compile: 'gloupIt <gloup3>'.
	self assert: lastReceivedAnnounce notNil.
	oldLastReceivedAnnounce := lastReceivedAnnounce.
	collector unsubscribe: self.
	settingsProviderClass compile: 'gloupItGloupIt <gloup3>'.
	self assert: lastReceivedAnnounce == oldLastReceivedAnnounce.
	settingsProviderClass removeSelector: #gloupIt.
	settingsProviderClass removeSelector: #gloupItGloupIt.
	self assert: collector isEmpty.
	collector release

! !

!PragmaCollectorTest methodsFor: 'test - pragma announcement' stamp: 'alain.plantec 4/13/2009 07:27'!
testPragmaUpdated
	| oldHandler collector |
	collector := self newCollectorForKeyword: #gloup5.
	settingsProviderClass compile: 'gloupIt <gloup5>'.
	self assert: (collector collected size = 1).
	oldHandler := collector collected first.
	collector whenChangedSend: #receiveAnnounce: to: self.
	lastReceivedAnnounce := nil.
	settingsProviderClass compile: 'gloupIt <gloup5> ^ true'.
	self
		assert: ((lastReceivedAnnounce isKindOf: PragmaAnnouncement)
				and: [lastReceivedAnnounce isHandlerUpdated]).
	self assert: (collector collected size = 1).
	self assert: (oldHandler == lastReceivedAnnounce handler).
	self assert: (collector collected first == lastReceivedAnnounce handler).
	collector unsubscribe: self.
	settingsProviderClass removeSelector: #gloupIt.
	self assert: collector isEmpty.
	collector release


	


! !

!PragmaCollectorTest methodsFor: 'test - system notifications' stamp: 'alain.plantec 3/30/2009 13:14'!
testRemovePragma
	| collector |
	"the requestor pragma holder"
	collector := self newCollectorForKeyword: #gloup8.
	settingsProviderClass compile: 'gloupIt <gloup8>'.
	self assert: (collector collected size = 1).
	settingsProviderClass removeSelector: #gloupIt.
	self assert: collector isEmpty.
	collector release


! !

!PragmaCollectorTest methodsFor: 'test - system notifications' stamp: 'alain.plantec 3/30/2009 13:14'!
testUpdatePragma
	| xxholder collector |
	collector := self newCollectorForKeyword: #gloup9.
	settingsProviderClass compile: 'gloupIt <gloup9>'.
	xxholder := collector detect: [:handler | handler selector = #gloupIt].
	self assert: xxholder notNil.
	settingsProviderClass compile: 'gloupIt <gloup9>'.
	self assert: (collector collect: [:handler | handler selector = #gloupIt]) size = 1.
	self assert: xxholder == (collector detect: [:handler | handler selector = #gloupIt]).
	settingsProviderClass removeSelector: #gloupIt.
	self assert: collector isEmpty.
	collector release


! !

TestCase subclass: #PragmaSettingDeclarationsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tests-System-Settings'!

!PragmaSettingDeclarationsTest methodsFor: 'test - settings reliability' stamp: 'alain.plantec 4/20/2009 08:53'!
testMenuItems
	(SystemMenuBrowser new collector collect: [:i | i value ]) do: [:i | self assert: i testIt description: i description]! !

!PragmaSettingDeclarationsTest methodsFor: 'test - menu assembling' stamp: 'alain.plantec 4/20/2009 14:33'!
testSystemMenuAssembling
	"Builds the world menu" 
	self assert: (((SettingMenuAssembler forKeywords: {SettingMenuAssembler worldMenuKeyword}) buildMenuStartingAt: #worldMenu) isKindOf: MenuMorph)! !

!PragmaSettingDeclarationsTest methodsFor: 'test - settings collecting' stamp: 'alain.plantec 4/20/2009 08:53'!
testSystemMenuItemCollect
	"collect all menu items, this is to test if all settings are consistent"
	SystemMenuBrowser new collector collect: [:i | i value ]! !

!PragmaSettingDeclarationsTest methodsFor: 'test - settings collecting' stamp: 'alain.plantec 4/20/2009 08:53'!
testSystemSettingCollect
	"collect all setting values, this is to test if all settings are consistent"
	SystemSettingBrowser new collector collect: [:i | i value value]! !

TestCase subclass: #PragmaSettingHardReset
	instanceVariableNames: 'settingsProviderClass settingsListenerClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tests-System-Settings'!

!PragmaSettingHardReset methodsFor: 'running' stamp: 'alain.plantec 4/9/2009 02:04'!
setUp
	super setUp.
	settingsProviderClass := Object
				subclass: #SETTINGSPROVIDERFORSETTINGSTESTS
				instanceVariableNames: ''
				classVariableNames: ''
				poolDictionaries: ''
				category: 'SETTINGSTESTS'.
	settingsProviderClass addClassVarName: #HTPPPORT.
	settingsProviderClass class compile: 'resetHTPPPORT HTPPPORT := nil'.
	settingsProviderClass class compile: 'hTPPPORT <' , self settingPragmaKeyword , '>
	^ HTPPPORT
		ifNil: [HTPPPORT := (SettingManager newSetting: ''Port'') default: 80]'.
	settingsListenerClass := Object
				subclass: #SETTINGSLISTENERFORSETTINGSTESTS
				instanceVariableNames: 'settingValue changeCount'
				classVariableNames: ''
				poolDictionaries: ''
				category: 'SETTINGSTESTS'.
	settingsListenerClass compile: 'initialize changeCount := 0'.
	settingsListenerClass compile: 'changeCount ^ changeCount'.
	settingsListenerClass compile: 'settingValueChanged changeCount := changeCount + 1'.
	settingsListenerClass compile: 'settingValueChanged: aValue settingValue := aValue'.
	settingsListenerClass compile: 'settingValue ^ settingValue'! !

!PragmaSettingHardReset methodsFor: 'test - setting hard reset' stamp: 'alain.plantec 4/6/2009 13:35'!
settingPragmaKeyword
	^ 'TESTSETTINGPRAGMA'! !

!PragmaSettingHardReset methodsFor: 'running' stamp: 'alain.plantec 4/6/2009 11:50'!
tearDown
	super tearDown.
	Smalltalk removeClassNamed: settingsProviderClass name.
	Smalltalk removeClassNamed: settingsListenerClass name.
	SystemOrganization removeSystemCategory: 'SETTINGSTESTS'.
! !

!PragmaSettingHardReset methodsFor: 'test - setting hard reset' stamp: 'alain.plantec 4/20/2009 08:51'!
testSettingHardReset
	| portSetting collector portSettingHandler listener |
	settingsProviderClass resetHTPPPORT.
	portSetting := settingsProviderClass hTPPPORT.
	self assert: portSetting value = 80.
	collector := SettingCollector new acceptableKeywords: {self settingPragmaKeyword}.
	portSettingHandler := collector detect: [:handler | handler value == portSetting].
	self assert: portSettingHandler notNil.
	self assert: portSettingHandler value == portSetting.
	"Declare listener interest to portSetting value changes"
	listener := settingsListenerClass new.
	portSetting whenChangedSend: #settingValueChanged to: listener.
	portSetting whenChangedSend: #settingValueChanged: to: listener.
	self assert: listener changeCount = 0.
	portSetting value: 8080.
	self assert: listener changeCount = 1.
	self assert: listener settingValue = portSetting value.
	portSettingHandler hardReset.
	self assert: settingsProviderClass hTPPPORT ~= portSetting.
	self assert: portSettingHandler value ~~ portSetting.
	self assert: portSettingHandler value == settingsProviderClass hTPPPORT.
	"hard reset has notified change to the default"
	self assert: listener changeCount = 2.
	self assert: listener settingValue ~= portSetting value.
	"returned to the default value"
	self assert: listener settingValue = 80.
	self assert: listener settingValue = settingsProviderClass hTPPPORT value.
	"old portSetting is dead"
	portSetting value: 8888.
	self assert: listener changeCount = 2.
	self assert: listener settingValue ~= portSetting value.
	"new portsetting is alive"
	self assert: listener settingValue = 80.
	portSetting := settingsProviderClass hTPPPORT.
	portSetting value: 8888.
	self assert: portSetting value = 8888.
	self assert: listener changeCount = 3.
	self assert: listener settingValue = portSetting value! !

TestCase subclass: #PragmaSettingListenerTest
	instanceVariableNames: 'settingsProviderClass localHTPPPORT portChanged portChangedCount settingsListenerClass localHTPPPORTSetting'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tests-System-Settings'!

!PragmaSettingListenerTest methodsFor: 'test - declaring interest' stamp: 'alain.plantec 4/6/2009 13:08'!
portChanged
	portChanged := portChanged not! !

!PragmaSettingListenerTest methodsFor: 'test - declaring interest' stamp: 'alain.plantec 4/9/2009 02:11'!
portChanged: aPortValue
	localHTPPPORT := aPortValue! !

!PragmaSettingListenerTest methodsFor: 'test - declaring interest' stamp: 'alain.plantec 4/6/2009 13:08'!
portChangedCount
	portChangedCount := portChangedCount + 1! !

!PragmaSettingListenerTest methodsFor: 'test - declaring interest' stamp: 'alain.plantec 4/21/2009 09:17'!
privatePortChanged: aSetting
	localHTPPPORTSetting := aSetting! !

!PragmaSettingListenerTest methodsFor: 'running' stamp: 'alain.plantec 4/6/2009 13:08'!
setUp
	super setUp.
	settingsProviderClass := Object subclass: #SETTINGSPROVIDERFORSETTINGSTESTS
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'SETTINGSTESTS'.
	settingsProviderClass addClassVarName: #HTPPPORT.
	settingsProviderClass class compile: 'resetHTPPPORT HTPPPORT := nil'.
	settingsProviderClass class compile: 'hTPPPORT
	^ HTPPPORT
		ifNil: [HTPPPORT := (SettingManager newSetting: ''Port'') default: 80]'.
	portChanged := false.
	portChangedCount := 0.
	settingsListenerClass := Object subclass: #SETTINGSLISTENERFORSETTINGSTESTS
		instanceVariableNames: 'changeCount'
		classVariableNames: ''
		poolDictionaries: ''
		category: 'SETTINGSTESTS'.
	settingsListenerClass compile: 'initialize changeCount := 0'.
	settingsListenerClass compile: 'changeCount ^ changeCount'.
	settingsListenerClass compile: 'settingChanged changeCount := changeCount + 1'
! !

!PragmaSettingListenerTest methodsFor: 'running' stamp: 'alain.plantec 4/6/2009 13:08'!
tearDown
	super tearDown.
	Smalltalk removeClassNamed: settingsProviderClass name.
	Smalltalk removeClassNamed: settingsListenerClass name.
	SystemOrganization removeSystemCategory: 'SETTINGSTESTS'.
! !

!PragmaSettingListenerTest methodsFor: 'test - declaring interest' stamp: 'alain.plantec 4/9/2009 02:15'!
testChangeInterestDeclaring
	| portSetting |
	settingsProviderClass resetHTPPPORT.
	portSetting := settingsProviderClass hTPPPORT.
	self assert: portSetting value = 80.
	"I declare my interest to portSetting changes"
	portSetting whenChangedSend: #portChanged: to: self.
	portSetting whenChangedSend: #portChanged to: self.
	portSetting whenChangedSend: #portChangedCount to: self.
	portSetting value: 8080.
	self assert: localHTPPPORT = 8080.
	self assert: portChanged.
	self assert: portChangedCount = 1.
	"Do no want to be notified with #portChanged anymore"
	portSetting forget: self selector: #portChanged.
	portSetting value: 8000.
	self assert: localHTPPPORT = 8000.
	self assert: portChanged.
	self assert: portChangedCount = 2.
	"Do no want to be notified at all"
	portSetting forget: self.
	portSetting value: 8888.
	self assert: portSetting value = 8888.
	self assert: localHTPPPORT = 8000.
	self assert: portChanged.
	self assert: portChangedCount = 2.
! !

!PragmaSettingListenerTest methodsFor: 'test - declaring interest' stamp: 'alain.plantec 4/9/2009 02:14'!
testChangeInterestGarbage
	| portSetting listener |
	settingsProviderClass resetHTPPPORT.
	portSetting := settingsProviderClass hTPPPORT.
	self assert: portSetting value = 80.
	"Declare listener interest to portSetting changes"
	listener := settingsListenerClass new.
	portSetting whenChangedSend: #settingChanged to: listener.
	self assert: (portSetting hasActionForEvent: portSetting changeEvent).
	portSetting value: 8080.
	self assert: listener changeCount = 1.
	listener := nil.
	Smalltalk garbageCollectMost.
	self assert: (portSetting hasActionForEvent: portSetting changeEvent) not
! !

!PragmaSettingListenerTest methodsFor: 'test - declaring interest' stamp: 'alain.plantec 4/21/2009 09:28'!
testPrivateChangeInterestDeclaring
	| portSetting |
	settingsProviderClass resetHTPPPORT.
	portSetting := settingsProviderClass hTPPPORT.
	self assert: portSetting value = 80.
	"I declare my interest to portSetting changes"
	portSetting whenPrivateChangedSend: #privatePortChanged: to: self.
	portSetting whenChangedSend: #portChanged: to: self.
	portSetting value: 8080.
	self assert: localHTPPPORT = 8080.
	self assert: localHTPPPORTSetting = portSetting.
	self assert: localHTPPPORTSetting value = localHTPPPORT.
	self assert: localHTPPPORTSetting value = portSetting value.
	self assert: localHTPPPORTSetting value = 8080.
	"Do no want to be notified with #portChanged: anymore"
	portSetting forget: self selector: #portChanged:.
	localHTPPPORTSetting := nil.
	localHTPPPORT := nil.
	portSetting value: 9090.
	self assert: localHTPPPORT = nil.
	self assert: localHTPPPORTSetting = portSetting.
	self assert: localHTPPPORTSetting value = portSetting value.
	self assert: localHTPPPORTSetting value = 9090.
	"Do no want to be notified with private version"
	portSetting privateForget: self.
	localHTPPPORTSetting := nil.
	portSetting value: 9090.
	self assert: localHTPPPORT = nil.
	self assert: localHTPPPORTSetting = nil.
	self assert: portSetting value = 9090.
	




! !
